/**
 * @file
 * Provides Intense loader.
 */

(function ($, Drupal, _win) {

  'use strict';

  var _id = 'intense';
  var _idOnce = _id;
  var _idBox = _id + '-box';
  var _intensed = 'intensed';
  var _gallery = _id + '-gallery';
  var _element = '.' + _gallery;
  var _mounted = _gallery + '--on';
  var _isIntensed = 'is-' + _intensed;
  var _elDefault = '[data-' + _gallery + ']';
  var _dataTrigger = '[data-' + _id + '-trigger]';
  var _elTrigger = _dataTrigger + ', .' + _id;
  var _elArrow = '.' + _intensed + '__arrow';
  var _elPrev = '.' + _intensed + '__prev';
  var _elNext = '.' + _intensed + '__next';
  var _visualyHidden = 'element-invisible';
  var _isHidden = 'is-hidden';
  var _intenseTimer = 0;
  var _add = 'addClass';
  var _remove = 'removeClass';
  var _loading = _id + '--loading';
  var _idClose = _id + '-close';
  var _btnClose = '.blazybox__close';
  var _body = 'body';
  var _blazyBox = Drupal.blazyBox || {};

  /**
   * Intense utility functions.
   */
  Drupal.intense = Drupal.intense || {
    btnClose: [],
    $box: [],
    $blazyBox: [],
    $gallery: [],
    $prev: [],
    $next: [],
    $arrow: [],
    $triggers: [],
    currentIndex: 0,
    index: 0,
    totalCount: 0,

    prepare: function () {
      var me = this;
      var body = $(_body);
      var arrow = body.find(_elArrow);

      if (me.count > 1 && !arrow.length) {
        body.append(Drupal.theme(_id));
      }

      me.$prev = me.$prev.length ? me.$prev : body.find(_elPrev);
      me.$next = me.$next.length ? me.$next : body.find(_elNext);

      if (me.$prev.length) {
        var prev = 'click.' + _id + 'Prev';
        body.off(prev).on(prev, _elPrev, function (e) {
          me.navigate(e, -1);
        });
      }

      if (me.$next.length) {
        var next = 'click.' + _id + 'Next';
        body.off(next).on(next, _elNext, function (e) {
          me.navigate(e, 1);
        });
      }
    },

    build: function (active, idx) {
      var me = this;
      var box;
      var i = idx || 0;

      if (me.$prev.length) {
        me.$prev[i === 0 || !active ? _add : _remove](_isHidden);
      }

      if (me.$next.length) {
        me.$next[i === me.count - 1 || !active ? _add : _remove](_isHidden);
      }

      _win.setTimeout(function () {
        me.check();
        box = me.$box;

        if (box.length) {
          box.addClass(_intensed);
          var cidbox = 'click.' + _idBox;
          box.off(cidbox).on(cidbox, me.closeOut.bind(me));
        }
      }, 101);
    },

    check: function () {
      var me = this;
      me.$box = $(_body).find('> figure:last');
    },

    toggle: function (visible, idx) {
      var me = this;

      $(_body)[visible ? _add : _remove](_isIntensed);

      // We have no events from the library, so have to use a delay.
      _win.setTimeout(function () {
        me.build(visible, idx);
      }, 101);
    },

    navigate: function (e, delta) {
      e.preventDefault();
      e.stopPropagation();

      var me = Drupal.intense;
      var blazyBox = me.$blazyBox;
      var box;

      var i = me.index + delta;
      var i2 = (i > me.count) ? me.count : i;

      i = (i < 0) ? 0 : i2;

      var launch = function () {
        // First close video, if any.
        if (blazyBox.length && !blazyBox.hasClass(_visualyHidden)) {
          var close = me.btnClose;
          if (close.length) {
            close.click();
          }
        }

        me.check();
        box = me.$box;

        if (box.length) {
          // Then close the existing zoomed image.
          var img = box.find('img');
          if (img.length) {
            img.click();
          }
        }

        // Finally always trigger a new one, if any.
        var link = $(me.$triggers[i]);
        if (link.length) {
          me.index = i;
          link[0].click();
          me.toggle(true, i);
        }
      };

      if (!e.detail || e.detail === 1) {
        launch();
      }
    },

    closeOut: function () {
      var me = this;
      me.toggle(false);
      me.$box = null;
      me.index = 0;
    },

    /**
     * Gets the current clicked item index.
     *
     * @param {HTMLElement} item
     *   The link item HTML element.
     *
     * @return {Int}
     *   The current clicked item index.
     */
    getIndex: function (item) {
      var me = this;
      var i = 0;
      $(me.$triggers).each(function (idx, elm) {
        if (elm === item) {
          i = idx;
          return i;
        }
      });
      return i;
    }
  };

  /**
   * Intense utility functions.
   *
   * @param {int} i
   *   The index of the current element.
   * @param {HTMLElement} elm
   *   The Intense gallery HTML element.
   */
  function process(i, elm) {
    var me = Drupal.intense;
    var $elm = $(elm);
    var hasMedia = $('.media--video', elm).length;
    var bbox;

    me.$gallery = $elm;
    me.$triggers = $elm.find(_elTrigger);
    me.count = me.$triggers.length;

    // Attaches Blazy video full screen.
    if (hasMedia && _blazyBox) {
      _blazyBox.attach();

      _win.setTimeout(function () {
        bbox = me.$blazyBox = _blazyBox.$el;
        me.btnClose = bbox.find(_btnClose);
        var cclose = 'click.' + _idClose;
        bbox.off(cclose).on(cclose, _btnClose, me.closeOut.bind(me));
      });
    }

    // Intensify images.
    Intense(me.$triggers);

    /**
     * Intense utility functions.
     *
     * @param {Event} e
     *   The event triggering the Intense.
     *
     * @return {bool}
     *   Returns a bailout if a dblclick detected, else passes through.
     */
    function triggerIntense(e) {
      e.preventDefault();

      // The e.target points to an IMG, and e.currentTarget (jQuery) to A tag.
      var target = $(e.currentTarget);
      var link = target.attr('href') ? target : target.closest(_elTrigger);
      var media = link.data('media') || false;
      var video = media && media.type === 'video';
      var delta = me.getIndex(link[0]);

      if (e.type === 'dblclick') {
        return false;
      }

      // We have no events from the library, so have to use a delay.
      _win.clearTimeout(_intenseTimer);
      _intenseTimer = _win.setTimeout(function () {

        // Build own video fullscreen as Intense is not for video.
        if (video && _blazyBox) {
          var url = link.attr('data-oembed-url') || link[0].href;
          _blazyBox.open(url);
        }

        me.index = delta;
        me.toggle(true, delta);
      });
      return true;
    }

    var clickCallback = function (e) {
      if (!e.detail || e.detail === 1) {
        var elems = $elm.find('.' + _loading);
        $.each(elems, function (x, el) {
          $(el).removeClass(_loading);
        });

        me.check();

        // @todo fixme, for some reasons, viewers were not cleanly removed.
        // if (me.$box.length > 1) {
        // me.$box.remove();
        // me.$box = null;
        // }
        triggerIntense(e);
      }
    };

    me.prepare();

    $elm.on('click.' + _id, _elTrigger, clickCallback);
    $elm.addClass(_mounted);
  }

  /**
   * Theme function for intense arrows.
   *
   * @return {HTMLElement}
   *   Returns a HTMLElement object.
   */
  Drupal.theme.intense = function () {
    var html;

    html = '<div class="intensed__arrow">';
    html += '<button class="intensed__prev" data-role="none">' + Drupal.t('Previous') + '</button>';
    html += '<button class="intensed__next" data-role="none">' + Drupal.t('Next') + '</button>';
    html += '</div>';

    return html;
  };

  /**
   * Attaches Intense behavior to HTML element.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.intense = {
    attach: function (context) {
      // Supports module or custom gallery with a CSS class.
      var sels = _elDefault + ',' + _element;
      var elms = $(context).find(sels);

      if (elms.length) {
        // Simplify and unify selectors for easy once.
        $.each(elms, function (i, el) {
          var $el = $(el);
          if (!$el.hasClass(_gallery)) {
            $el.addClass(_gallery);
          }
        });
        $(_element, context).once(_idOnce, process);
      }
    }
  };

})(jQuery, Drupal, this);
